/* ************************************************************************

   Copyright: 2010 by Michael Olbrich <m.olbrich@pengutronix.de>

   License:
     To the extent possible under law, Pengutronix e.K. has
     waived all copyright and related or neighboring rights to
     JSON-DBus-Bridge Demo Code.

************************************************************************ */

/* ************************************************************************

#asset(calculator/*)

************************************************************************ */

/**
 * This is the main application class of your custom application "calculator"
 */
qx.Class.define("calculator.Application",
{
  extend : qx.application.Standalone,



  /*
  *****************************************************************************
     MEMBERS
  *****************************************************************************
  */

  members :
  {
    __edit: null,
    __win: null,
    __last: null,
    __lastCmd: null,
    __clear: false,
    __rpc: null,

    /**
     * This method contains the initial application code and gets called 
     * during startup of the application
     * 
     * @lint ignoreDeprecated(alert)
     */
    main : function()
    {
      // Call super class
      this.base(arguments);

      // Enable logging in debug variant
      if (qx.core.Variant.isSet("qx.debug", "on"))
      {
        // support native logging capabilities, e.g. Firebug for Firefox
        qx.log.appender.Native;
        // support additional cross-browser console. Press F7 to toggle visibility
        qx.log.appender.Console;
      }

      /*
      -------------------------------------------------------------------------
        Below is your actual application code...
      -------------------------------------------------------------------------
      */

      // Document is the application root
      var doc = this.getRoot();
      doc.setBackgroundColor("#ffffff");

      var back = new qx.ui.basic.Image("calculator/back.png");
      back.setScale(true);
      doc.add(back, {left:0, top: 0});

      var win = new qx.ui.window.Window("Calculator");
      this.__win = win;
      doc.add(win, {left: 50, top: 50});

      var winLayout = new qx.ui.layout.Grid();
      winLayout.setSpacing(10);
      win.setLayout(winLayout);
      win.setShowStatusbar(true);
      win.open();

      this.__edit = new qx.ui.form.TextField();
      this.__edit.setAppearance("calc-edit");
      win.add(this.__edit, {row: 0, column: 0, colSpan: 5 });
      this.__edit.setReadOnly(true);

      var app = this;
      var makeButton = function(text, r, c)
      {
        var button = new qx.ui.form.Button(text);
        win.add(button, {row: r, column: c });
        button.addListener("execute", function(e) {
                            this.__doKey(text);
                           }, app);
        button.setAppearance("calc-button");
      }
      makeButton("1", 4, 0);
      makeButton("2", 4, 1);
      makeButton("3", 4, 2);
      makeButton("4", 3, 0);
      makeButton("5", 3, 1);
      makeButton("6", 3, 2);
      makeButton("7", 2, 0);
      makeButton("8", 2, 1);
      makeButton("9", 2, 2);
      makeButton("0", 5, 0);

      var clearButton = new qx.ui.form.Button("C");
      win.add(clearButton, {row: 1, column: 0});
      clearButton.addListener("execute", this.__clearAll, this);
      clearButton.setAppearance("calc-button");

      var makeOpp = function(text, func, r, c, span)
      {
        if (span == null)
          span = 1;
        var button = new qx.ui.form.Button(text);
        win.add(button, {row: r, column: c, colSpan: span });
        button.addListener("execute", function(e) {
                            app.__doExec(func);
                           }, app);
        button.setAppearance("calc-button");
      }

      makeOpp("=", null, 5, 4);
      makeOpp("+", "Add", 4, 4);
      makeOpp("-", "Sub", 3, 4);
      makeOpp("*", "Multiply", 2, 4);
      makeOpp("/", "Divide", 1, 4);

      this.__rpc = new qx.io.remote.Rpc("http://localhost:8080/rpc", "com.pengutronix.jdb.Calculator|/");

    },
    __doKey : function(key)
    {
      var value;
      this.__win.setStatus("");
      if (this.__clear) {
        value = "";
        this.__clear = false;
      }
      else {
        value = this.__edit.getValue();
        if (value == null)
          value = "";
      }
      this.__edit.setValue(value + key);
    },
    __doExec : function(func)
    {
      this.__win.setStatus("");
      var current = parseInt(this.__edit.getValue());
      if (current == null)
        current = 0;
      if ((this.__last == null) || (this.__lastCmd == null)) {
        this.__last = current;
      }
      else {
        try {
          this.__last = this.__rpc.callSync("com.pengutronix.jdb.Calculator." + this.__lastCmd,
                                            "ii", this.__last, current);
          this.__edit.setValue("" + this.__last);
        }
        catch (error) {
          this.__clearAll();
          this.__win.setStatus(error);
          return;
        }
      }
      this.__lastCmd = func;
      this.__clear = true;
    },
    __clearAll : function()
    {
      this.__last = null;
      this.__lastCmd = null;
      this.__edit.setValue("");
    }
  }
});
